/* BdfServer - Copyright (c) 2015-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.ui.components;

import fr.exemole.bdfserver.api.BdfServerConstants;
import fr.exemole.bdfserver.api.ui.CommentUi;
import fr.exemole.bdfserver.api.ui.DataUi;
import fr.exemole.bdfserver.api.ui.FieldUi;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.api.ui.options.InputOption;
import fr.exemole.bdfserver.api.ui.options.ValueOption;
import fr.exemole.bdfserver.tools.ui.UiUtils;
import fr.exemole.bdfserver.tools.ui.options.UiOptionsUtils;
import java.text.ParseException;
import java.util.function.Consumer;
import net.fichotheque.FichothequeConstants;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.include.ExtendedIncludeKey;
import net.fichotheque.include.IncludeKey;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.html.TrustedHtmlFactory;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.DomMessages;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public final class UiDOMUtils {

    private UiDOMUtils() {

    }

    public static CommentUi readCommentUi(Element element, TrustedHtmlFactory trustedHtmlFactory, MessageHandler messageHandler, String xpath) {
        String name = element.getAttribute("name");
        if (name.length() == 0) {
            DomMessages.emptyAttribute(messageHandler, xpath, "name");
            return null;
        }
        CommentUiBuilder commentUiBuilder = CommentUiBuilder.init(trustedHtmlFactory, name);
        String locationString = element.getAttribute("location");
        if (locationString.length() != 0) {
            int location = UiUtils.maskStringToInt(locationString);
            commentUiBuilder.setLocation(location);
        }
        DOMUtils.readChildren(element, new CommentUiConsumer(commentUiBuilder, messageHandler));
        return commentUiBuilder.toCommentUi();
    }

    public static DataUi readDataUi(Element element, MessageHandler messageHandler, String xpath) {
        String name = element.getAttribute("name");
        if (name.length() == 0) {
            DomMessages.emptyAttribute(messageHandler, xpath, "name");
            return null;
        }
        String type = element.getAttribute("type");
        if (type.length() == 0) {
            DomMessages.emptyAttribute(messageHandler, xpath, "name");
            return null;
        }
        DataUiBuilder dataUiBuilder = DataUiBuilder.init(name, type);
        DOMUtils.readChildren(element, new DataUiConsumer(dataUiBuilder, messageHandler));
        return dataUiBuilder.toDataUi();
    }

    public static FieldUi readFieldUi(Element element, MessageHandler messageHandler, String xpath) {
        String fieldKeyString = element.getAttribute("field-key");
        if (fieldKeyString.length() == 0) {
            DomMessages.emptyAttribute(messageHandler, xpath, "field-key");
            return null;
        }
        FieldKey fieldKey;
        try {
            fieldKey = FieldKey.parse(fieldKeyString);
        } catch (ParseException pe) {
            DomMessages.wrongAttributeValue(messageHandler, xpath, "field-key", fieldKeyString);
            return null;
        }
        FieldUiBuilder fieldUiBuilder = FieldUiBuilder.init(fieldKey);
        DOMUtils.readChildren(element, new FieldUiConsumer(fieldUiBuilder, messageHandler));
        return fieldUiBuilder.toFieldUi();
    }

    public static IncludeUi readSubsetIncludeUi(Element element_xml, MessageHandler messageHandler, String xpath) {
        boolean master = false;
        SubsetKey subsetKey = null;
        String mode = "";
        int poids = -1;
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            if (liste.item(i).getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("master")) {
                    master = true;
                } else if (tagName.equals("subset")) {
                    String subsetKeyString = XMLUtils.getData(element);
                    if (subsetKeyString.length() == 0) {
                        DomMessages.emptyElement(messageHandler, "subset");
                        return null;
                    }
                    try {
                        subsetKey = SubsetKey.parse(subsetKeyString);
                    } catch (java.text.ParseException pe) {
                        DomMessages.wrongElementValue(messageHandler, "subset", subsetKeyString);
                        return null;
                    }
                } else if (tagName.equals("poids")) {
                    String poidsString = XMLUtils.getData(element);
                    if (poidsString.length() == 0) {
                        DomMessages.emptyElement(messageHandler, "poids");
                        return null;
                    }
                    try {
                        poids = Integer.parseInt(poidsString);
                    } catch (NumberFormatException nfe) {
                        DomMessages.wrongElementValue(messageHandler, "poids", poidsString);
                        return null;
                    }
                } else if (tagName.equals("mode")) {
                    mode = XMLUtils.getData(element);
                    if (mode.length() > 0) {
                        try {
                            StringUtils.checkTechnicalName(mode, false);
                        } catch (ParseException pe) {
                            DomMessages.wrongElementValue(messageHandler, "mode", mode);
                            return null;
                        }
                    }
                }
            }
        }
        if (subsetKey == null) {
            DomMessages.emptyElement(messageHandler, xpath);
            return null;
        }
        try {
            ExtendedIncludeKey extendedIncludeKey = ExtendedIncludeKey.newInstance(IncludeKey.newInstance(subsetKey, mode, poids), master);
            IncludeUiBuilder includeUiBuilder = IncludeUiBuilder.initSubset(extendedIncludeKey);
            DOMUtils.readChildren(element_xml, new IncludeUiConsumer(includeUiBuilder, messageHandler));
            return includeUiBuilder.toIncludeUi();
        } catch (IllegalArgumentException iae) {
            DomMessages.wrongElementValue(messageHandler, xpath, subsetKey.getKeyString());
            return null;
        }
    }

    public static IncludeUi readSpecialIncludeUi(Element element, MessageHandler messageHandler, String xpath) {
        String name = element.getAttribute("name");
        if (name.length() == 0) {
            name = element.getAttribute("include-key");
        }
        if (name.length() == 0) {
            DomMessages.emptyAttribute(messageHandler, xpath, "name");
            return null;
        }
        IncludeUiBuilder includeUiBuilder;
        if (name.equals("association")) {
            ExtendedIncludeKey extendedIncludeKey = ExtendedIncludeKey.build("association");
            includeUiBuilder = IncludeUiBuilder.initSubset(extendedIncludeKey);
        } else {
            try {
                name = FichothequeConstants.checkSpecialIncludeName(name);
                includeUiBuilder = IncludeUiBuilder.initSpecial(name);
            } catch (IllegalArgumentException iae) {
                DomMessages.wrongAttributeValue(messageHandler, xpath, "name", name);
                return null;
            }
        }
        DOMUtils.readChildren(element, new IncludeUiConsumer(includeUiBuilder, messageHandler));
        return includeUiBuilder.toIncludeUi();
    }


    private static class FieldUiConsumer implements Consumer<Element> {

        private final FieldUiBuilder fieldUiBuilder;
        private final MessageHandler messageHandler;

        private FieldUiConsumer(FieldUiBuilder fieldUiBuilder, MessageHandler messageHandler) {
            this.fieldUiBuilder = fieldUiBuilder;
            this.messageHandler = messageHandler;
        }

        @Override
        public void accept(Element element) {
            String tagName = element.getTagName();
            switch (tagName) {
                case "status":
                    String status = DOMUtils.readSimpleElement(element);
                    try {
                        fieldUiBuilder.setStatus(status);
                    } catch (IllegalArgumentException iae) {

                    }
                    break;
                case "input": {
                    InputOption inputOption = UiOptionsUtils.readInputOption(element, messageHandler);
                    if (inputOption != null) {
                        fieldUiBuilder.setInputOption(inputOption);
                    }
                    break;
                }
                case "value": {
                    if (StringUtils.isTrue(element.getAttribute("mandatory"))) {
                        fieldUiBuilder.setStatus(BdfServerConstants.MANDATORY_STATUS);
                    }
                    ValueOption valueOption = UiOptionsUtils.readValueOption(element, messageHandler);
                    if (valueOption != null) {
                        fieldUiBuilder.setValueOption(valueOption);
                    }
                    break;
                }
                case "attr": {
                    AttributeUtils.readAttrElement(fieldUiBuilder.getAttributesBuilder(), element);
                    break;
                }
            }
        }

    }


    private static class IncludeUiConsumer implements Consumer<Element> {

        private final IncludeUiBuilder includeUiBuilder;
        private final MessageHandler messageHandler;


        private IncludeUiConsumer(IncludeUiBuilder includeUiBuilder, MessageHandler messageHandler) {
            this.includeUiBuilder = includeUiBuilder;
            this.messageHandler = messageHandler;
        }

        @Override
        public void accept(Element element) {
            String tagName = element.getTagName();
            switch (tagName) {
                case "status":
                    String status = DOMUtils.readSimpleElement(element);
                    try {
                        includeUiBuilder.setStatus(status);
                    } catch (IllegalArgumentException iae) {

                    }
                    break;
                case "custom": {
                    DOMUtils.readChildren(element, new CustomConsumer(includeUiBuilder, messageHandler));
                    break;
                }
                case "input": {
                    InputOption inputOption = UiOptionsUtils.readInputOption(element, messageHandler);
                    if (inputOption != null) {
                        includeUiBuilder.setInputOption(inputOption);
                    }
                    break;
                }
                case "value": {
                    if (StringUtils.isTrue(element.getAttribute("mandatory"))) {
                        includeUiBuilder.setStatus(BdfServerConstants.MANDATORY_STATUS);
                    }
                    ValueOption valueOption = UiOptionsUtils.readValueOption(element, messageHandler);
                    if (valueOption != null) {
                        includeUiBuilder.setValueOption(valueOption);
                    }
                    break;
                }
                case "limit": {
                    if (StringUtils.isTrue(element.getAttribute("nonew"))) {
                        includeUiBuilder.setStatus(BdfServerConstants.OBSOLETE_STATUS);
                    }
                    break;
                }
                case "attr": {
                    AttributeUtils.readAttrElement(includeUiBuilder.getAttributesBuilder(), element);
                    break;
                }
            }
        }

    }


    private static class CommentUiConsumer implements Consumer<Element> {

        private final CommentUiBuilder commentUiBuilder;
        private final MessageHandler messageHandler;

        private CommentUiConsumer(CommentUiBuilder commentUiBuilder, MessageHandler messageHandler) {
            this.commentUiBuilder = commentUiBuilder;
            this.messageHandler = messageHandler;
        }

        @Override
        public void accept(Element element
        ) {
            String tagName = element.getTagName();
            switch (tagName) {
                case "html":
                case "lib": {
                    String langString = element.getAttribute("xml:lang");
                    if (langString.length() > 0) {
                        try {
                            Lang lang = Lang.parse(langString);
                            String htmlString = XMLUtils.getRawData(element);
                            commentUiBuilder.putHtml(lang, htmlString);
                        } catch (ParseException pe) {
                            DomMessages.wrongLangAttribute(messageHandler, tagName, langString);
                        }
                    } else {
                        DomMessages.emptyAttribute(messageHandler, tagName, "xml:lang");
                    }
                    break;
                }
                case "attr": {
                    AttributeUtils.readAttrElement(commentUiBuilder.getAttributesBuilder(), element);
                    break;
                }
                default:
                    DomMessages.unknownTagWarning(messageHandler, tagName);
            }
        }

    }


    private static class DataUiConsumer implements Consumer<Element> {

        private final DataUiBuilder dataUiBuilder;
        private final MessageHandler messageHandler;

        private DataUiConsumer(DataUiBuilder dataUiBuilder, MessageHandler messageHandler) {
            this.dataUiBuilder = dataUiBuilder;
            this.messageHandler = messageHandler;
        }

        @Override
        public void accept(Element element) {
            String tagName = element.getTagName();
            switch (tagName) {
                case "param": {
                    String name = element.getAttribute("name");
                    if (name.length() > 0) {
                        String value = DOMUtils.readSimpleElement(element);
                        dataUiBuilder.getExternalSourceDefBuilder().addParam(name, value);
                    }
                    break;
                }
                case "label": {
                    try {
                        Label label = LabelUtils.readLabel(element);
                        if (label != null) {
                            dataUiBuilder.putLabel(label);
                        }
                    } catch (ParseException ile) {
                        DomMessages.wrongLangAttribute(messageHandler, tagName, element.getAttribute("xml:lang"));
                    }
                    break;
                }
                case "attr": {
                    AttributeUtils.readAttrElement(dataUiBuilder.getAttributesBuilder(), element);
                    break;
                }
                default:
                    DomMessages.unknownTagWarning(messageHandler, tagName);
            }
        }

    }


    private static class CustomConsumer implements Consumer<Element> {

        private final IncludeUiBuilder includeUiBuilder;
        private final MessageHandler messageHandler;


        private CustomConsumer(IncludeUiBuilder includeUiBuilder, MessageHandler messageHandler) {
            this.includeUiBuilder = includeUiBuilder;
            this.messageHandler = messageHandler;
        }

        @Override
        public void accept(Element element) {
            String tagName = element.getTagName();
            switch (tagName) {
                case "label":
                case "lib": {
                    try {
                        Label label = LabelUtils.readLabel(element);
                        if (label != null) {
                            includeUiBuilder.putLabel(label);
                        }
                    } catch (ParseException ile) {
                        DomMessages.wrongLangAttribute(messageHandler, tagName, element.getAttribute("xml:lang"));
                    }
                    break;
                }
                default:
                    DomMessages.unknownTagWarning(messageHandler, tagName);
            }
        }

    }

}
